/*   XN[  Ver 0.00   */
/*                          */
/*                          */
/*   by Atsushi   98/3/20   */


package game.awt;

import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Enumeration;
import java.util.Vector;
import game.awt.event.OffscreenEvent;
import game.awt.event.OffscreenListener;


public class Screen extends Component
{
	/*   ---   Field   ---   */

	private Image offscreen;

	private Vector offscreenListener;

	/*   ---   Constructor   ---   */

	public Screen()
	{
		offscreenListener=new Vector();
		
		enableEvents(AWTEvent.COMPONENT_EVENT_MASK);
		enableEvents(AWTEvent.FOCUS_EVENT_MASK);
		enableEvents(AWTEvent.KEY_EVENT_MASK);
		enableEvents(AWTEvent.MOUSE_EVENT_MASK);
		enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
		
		addComponentListener(new ResizeInformant(this));
	}

	/*   ---   obt@   ---   */

	public Image getOffscreenImage()
	{
		return offscreen;
	}

	public Graphics getOffscreenGraphics()
	{
		return offscreen.getGraphics();
	}

	/*   ---   Cxg   ---   */

	public void addOffscreenListener(OffscreenListener listener)
	{
		offscreenListener.addElement(listener);
	}

	public void removeOffscreenListener(OffscreenListener listener)
	{
		offscreenListener.removeElement(listener);
	}

	private void fireOffscreenChanged(OffscreenEvent event)
	{
		for (Enumeration enum=offscreenListener.elements();
		                 enum.hasMoreElements();)
		{
			((OffscreenListener)(enum.nextElement())).offscreenChanged(event);
		}
	}

	/*   ---   `揈   ---   */

	public void paint(Graphics g)
	{
		if (offscreen==null) initOffscreen();
		
		paintOffscreen(g);
	}

	public void update(Graphics g)
	{
		paint(g);
	}

	public void paintOffscreen(Graphics g)
	{
		g.drawImage(offscreen,0,0,null);
	}

	private void initOffscreen()
	{
		Dimension size;
	
		size=getSize();
		if (size.width==0 && size.height==0) return;
		
		offscreen=createImage(size.width,size.height);
		
		fireOffscreenChanged(new OffscreenEvent(this,offscreen));
	}

	/*   ---   Cxg   ---   */

	private class ResizeInformant extends ComponentAdapter
	{
		/*   ---   Field   ---   */
	
		private Screen screen;
	
		/*   ---   Constructor   ---   */
	
		public ResizeInformant(Screen screen)
		{
			this.screen=screen;
		}
	
		/*   ---   TCY   ---   */
	
		public void componentResized(ComponentEvent event)
		{
			screen.initOffscreen();
		}
	}

	/*   ---   Method  Override   ---   */

	public void invalidate()
	{
		super.invalidate();
		
		if (offscreen!=null) offscreen.flush();
		offscreen=null;
	}

	/*   ---   Finalizer   ---   */

	public void finalize() throws Throwable
	{
		if (offscreen!=null) offscreen.flush();
		
		super.finalize();
	}
}
