/*   nCXRALO  Ver 0.20   */
/*                                    */
/*                                    */
/*   by Atsushi              98/4/2   */


package game.ranking;

import java.io.*;
import java.net.URL;
import java.text.ParseException;
import java.util.*;
import game.ranking.RankingClient;
import game.ranking.RankingRecord;


public class Ranking extends Object implements Runnable
{
	/*   ---   萔   ---   */

	public final static int NO_RANKING=-1;

	private final static int ASCENDING = 1;
	private final static int DESCENDING=-1;

	/*   ---   Field   ---   */

	private RankingClient client;
	private Thread connection=null;
	private Stack entries;

	private RankingRecord record[];
	private int sort;

	private boolean stop;
	private String error;

	/*   ---   Constructor   ---   */

	public Ranking(URL host)
	{
		client=new RankingClient(host);
		entries=new Stack();
		
		error=null;
	}

	/*   ---   Gg[   ---   */

	private class Entry extends Object
	{
		/*   ---   Field   ---   */
	
		private double score;
		private String name;
		private String extension;
	
		/*   ---   Constructor   ---   */
	
		public Entry(double score,String name)
		{
			this.score    =score;
			this.name     =name.trim();
			this.extension=null;
		}
	
		public Entry(double score,String name,String extension)
		{
			this.score    =score;
			this.name     =name.trim();
			this.extension=extension.trim();
		}
	
		/*   ---   l   ---   */
	
		public double getScore()
		{
			return score;
		}
	
		public String getName()
		{
			return name;
		}
	
		public String getExtension()
		{
			return extension;
		}
	}

	/*   ---   ύX   ---   */

	public int getRank(double score)
	{
		for (int i=0;i<=record.length-1;i++)
		{
			if (record[i]==null || score*sort>record[i].getScore()*sort)
			{
				return i+1;
			}
		}
		
		return NO_RANKING;
	}

	public void put(double score,String name)
	{
		entries.push(new Entry(score,name));
		synchronized (this) {notify();}
	}

	public void put(double score,String name,String extension)
	{
		entries.push(new Entry(score,name,extension));
		synchronized (this) {notify();}
	}

	/*   ---   擾   ---   */

	public RankingRecord[] getRecords()
	{
		return record;
	}

	public double getHighScore()
	{
		if (record==null || record[0]==null)
		{
			return Double.NaN;
		}
		else
		{
			return record[0].getScore();
		}
	}

	public String getTop()
	{
		if (record==null || record[0]==null)
		{
			return null;
		}
		else
		{
			return record[0].getName();
		}
	}

	/*   ---   ڑ   ---   */

	public boolean isConnecting()
	{
		return (record==null);
	}

	/*   ---   G[   ---   */

	public boolean isError()
	{
		return (error!=null);
	}

	public String getErrorMessage()
	{
		return error;
	}

	/*   ---   s   ---   */

	public void startConnection()
	{
		if (connection==null)
		{
			connection=new Thread(this,"RankingClient");
			connection.start();
		}
		stop=false;
	}

	public void run()								// connection
	{
		Entry entry;
		String str;
	
		error=null;
		try {
			if (record==null)
			{
				initRecords(client.getHeader());
				setRecords(client.list());
			}
			
			while (true)
			{
				if (stop) break;
				
				if (entries.isEmpty())
				{
					synchronized (this)
					{
						try {wait();}
						catch (InterruptedException e) {}
					}
				}
				
				if (stop) break;
				
				entry=((Entry)(entries.pop()));
				if (entry.getExtension()==null)
				{
					putRecord(client.put(entry.getScore(),entry.getName()));
				}
				else
				{
					putRecord(client.put(entry.getScore(),entry.getName(),
					                     entry.getExtension()));
				}
				setRecords(client.list());
			}
		}
		catch (Exception e)
		{
			str=e.getClass().getName();
			str=str.substring(str.lastIndexOf(".")+1);
			error=(e.getMessage()==null ? str : str+" : "+e.getMessage());
		}
		connection=null;
	}

	public void stopConnection()
	{
		if (connection!=null)
		{
			stop=true;
			synchronized (this) {notify();}
		}
	}

	/*   ---   ύX   ---   */

	private void initRecords(String header)
	{
		StringTokenizer tokenizer;
		String str;
		int length;
	
		tokenizer=new StringTokenizer(header);
		
		length=Integer.parseInt(tokenizer.nextToken());
		str=tokenizer.nextToken();
		if      (str.equals("+")) sort=ASCENDING;
		else if (str.equals("-")) sort=DESCENDING;
		
		record=new RankingRecord[length];
	}

	private void setRecords(RankingRecord newRecord[])
	{
		for (int i=0;i<=newRecord.length-1;i++)
		{
			record[i]=newRecord[i];
		}
	}

	private void putRecord(RankingRecord newRecord)
	{
		int rank;
	
		rank=getRank(newRecord.getScore());
		if (rank!=NO_RANKING)
		{
			for (int i=record.length-1;i>=(rank-1)+1;i--)
			{
				record[i]=record[i-1];
			}
			record[rank-1]=newRecord;
		}
	}
}
